/******************************* H HEADER FILE ******************************
**
** Project		: LCD Library (4-bit Mode)
** Filename		: lcd.h
** Chip Type	: Any AVR
** Clock MHz	: 
** Compiler		: AVR GCC
**
*****************************************************************************
** 
** Copyright (c) 2006, Andrew Nickson, modified by Daniel Berg 2007
** All rights reserved.
**
** You are free to use this code as you see fit, a mention would be nice and any
** feedback.
** Email: andy@re-mastered.co.uk 
** Web:   www.re-mastered.co.uk
** 
*****************************************************************************
**
**	Currently this library only supports the LCD in 4-bit mode with the data lines on the
**	LOWER 4 bits. i.e :   
**
**	PORTx.0	=	LCD_BIT4		// mod. LCD_BIT4 is now on PORTB2 etc.
**	PORTx.1	=	LCD_BIT5
**	PORTx.2	=	LCD_BIT6
**	PORTx.3	=	LCD_BIT7
**	
**	Also the control (E, RS and RW) must be on the same port. This will be modified one day!
**
**	At the moment the RW line does not do anything, so you can just tie this low
**	on the LCD and then only require 6 IO's to control the LCD. Or write some functions
**	to read the busy properly or read any memory addresses.
**
*****************************************************************************/

#ifndef _LCD_H_
#define _LCD_H_

#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>

//#define LCD_LINES           2     // number of visible lines of the display
//#define LCD_DISP_LENGTH    16     // visibles characters per line of the display

#define LCD_PORT		PORTB	// Port which the LCD is connected
#define LCD_DDR			DDRB	// Data Direction Register (should be for the port above)
#define LCD_ENABLE		0		// Enable Pin
#define LCD_RS			1		// Register Select Pin
#define LCD_RW			0		// Read/Write Pin //not used!
#define LCD_SPARE		0		// Spare port Pin not used by LCD 

//macros
#define SET_E		LCD_PORT |= (1<<LCD_ENABLE);	//E	 = 1
#define CLEAR_E		LCD_PORT &= ~(1<<LCD_ENABLE);	//E	 = 0
#define SET_RS		LCD_PORT |= (1<<LCD_RS);		//RS = 1
#define CLEAR_RS	LCD_PORT &= ~(1<<LCD_RS);		//RS = 0


//#define SET_RW		LCD_PORT |= (1<<LCD_RW);		//RW = 1
//#define CLEAR_RW	LCD_PORT &= ~(1<<LCD_RW);		//RW = 0

#define LINE1		0x00
#define LINE2		0x40

typedef unsigned char byte;

void lcd_init(void);
void lcd_cls(void);
void lcd_home(void);
void lcd_cursor_on(void);
void lcd_cursor_off(void);
void lcd_disp_on(void);
void lcd_disp_off(void);
void lcd_goto(byte mesto);
void lcd_putch(byte data);
void lcd_putstr(byte *data);
void lcd_printhex(byte i);
void lcd_printint(unsigned int x);

#endif
