/** 
	Program fr kontroll av en hemmabyggd AZ/EL rotor
	
	Avsett fr anvndning av ett stlldon och rotor som genererar
	ett godtyckligt antal pulser per grad.
	
	
	
	Copyright 2007 Daniel Berg
	
 	This file is part of SatFinder.
 
    SatFinder is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SatFinder.  If not, see <http://www.gnu.org/licenses/>.
	
	
	
*/
#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <math.h>


#include "avr_lcd.h"
#include "knappanalys.h"
#include "Drivers_EL_AZ_local.h"


#define UP 		1
#define DOWN 	0
#define RIGHT	1
#define LEFT 	0

#define RIGHT_B 0
#define LEFT_B 	1
#define UP_B 	2
#define DOWN_B 	3
#define MENU_B 	5



#define PRINT_PULSES_ON 1
#define PRINT_PULSES_OFF 0

/* Saxat ur databladet (och aningen justerat) */
#define BAUD 600
#define MYUBRR F_CPU/16/BAUD-1

/* Stopptid fr stlldonet (tid det tar fr stlldonet att stanna frn
   det att strmmen stngts av */
#define STOPPTID 0.7		// sekunder
#define STOPPTID_M STOPPTID/0.032768

//#define USART_debug

//volatile uint16_t el_pulses;
//volatile uint8_t el_direction;
//volatile uint16_t az_pulses;
//volatile uint8_t az_direction;
//volatile uint16_t el_target;
//volatile uint16_t az_target;
//volatile uint16_t az_min;
//volatile uint16_t az_max;
//volatile uint16_t el_min;
//volatile uint16_t el_max;
//volatile uint16_t az_zero;
//volatile uint16_t az_360;
//volatile uint16_t el_zero;
//volatile uint16_t el_90;

////volatile uint8_t el_is_running;
//volatile uint8_t az_is_running;
volatile uint8_t NEW_USART_MSG;
//volatile uint8_t Stoppfaktor_el;
//volatile uint8_t Stoppfaktor_az;
//volatile uint8_t TCNT0H;
//volatile uint8_t OCR0A_high;
//volatile uint8_t latest_stop_pulses_up;
//volatile uint8_t latest_stop_pulses_down;
//volatile uint8_t latest_stop_pulses_az;
volatile uint8_t Ange_verklig_position;


volatile uint8_t TX_buffer[12];				// Vi gr rum fr det meddelande som skall sndas
volatile uint8_t TX_index;			
volatile uint8_t RX_buffer[13];				// "inkorg"






/** 
	Initieringsmojnger
*/
void io_init(void) {
	
	// aktivera interna pull-ups
	PORTC = (_BV(RIGHT_B) | _BV(LEFT_B) | _BV(UP_B) | _BV(DOWN_B) | _BV(MENU_B));
	//PORTD = (_BV(2) | _BV(3));					// skall nog ersttas av externa vad det lider
	
	EL_init_driver();
	AZ_init_driver();
	
	
	/* Starta UART med frdefininerad baudrate, saxat ur databladet */
	/* Sjlva avbrotten r inte nnu aktiverade */
	UBRR0H = ((MYUBRR)>>8);
	UBRR0L = MYUBRR;
	/* Enable receiver and transmitter */
	UCSR0B = _BV(RXEN0) | _BV(TXEN0);
	
	/* Inga meddelanden har nnu tagits emot */
	NEW_USART_MSG = 0;
	
	/* Vi brjar med att returnera den verkliga positionen till PC */
	Ange_verklig_position = 1;
	
	// Aktivera avbrott
	sei();
	
}

/**
	Starta sndning av meddelande
	(aktivera interrupt)
*/
static inline void Send_message(void) {
	TX_index = 0;
	// Aktivera avbrott
	UCSR0B |= _BV(UDRIE0);
}

void double_to_string (double value, char * num_string, const uint8_t decimaler) {
int8_t divisioner = 0;
uint8_t index = 0;

	// lgg till ett mjligt minustecken
	if (value < 0) {
		num_string[index++] = '-';
		value = -value;			// gr talet positivt!
	}
	
	// dividera med 10 tills talet r mindre n ett
	while (value >= 1) {
		value = value / 10;
		divisioner++;
	}

	// magiska rader efter id hittad p internet
	while (divisioner != (0-decimaler)) {
		if (divisioner == 0) num_string[index++] = '.';
		divisioner--;
		value = value * 10.0;
		num_string[index] = (char) (0x30 + (int) value);
		value = value - (double) (num_string[index++] - 0x30);
	}
	num_string[index] = 0;
}



/**
	Ange ny AZ-vilkel. Anrop av denna rutin verstter en nskad vinkel till
	ett heltal pulser.
*/
uint16_t AZ_angle_to_pulses (double vinkel) {
	return (uint16_t) (((vinkel / 360.0) * (AZ_get_high() - AZ_get_low())) + AZ_get_low());
}

/**
	Ange ny EL-vinkel. Anrop av denna rutin verstter en nskad vinkel till
	ett heltal pulser.
*/
uint16_t EL_angle_to_pulses (double vinkel) {
	return (uint16_t) (((vinkel / 90.0) * (EL_get_high() - EL_get_low())) + EL_get_low());
}

/* 
	Rkna om pulser till grader, EL
*/
double pulses_to_angle_el (uint16_t pulses) {
	return ((double) ((int16_t) pulses - (int16_t) EL_get_low())) / ((double) (EL_get_high() - EL_get_low())) * 90.0;
}

/*
	Rkna om pulser till grader, AZ
*/
double pulses_to_angle_az (uint16_t pulses) {
	return ((double) ((int16_t) pulses - (int16_t) AZ_get_low())) / ((double) (AZ_get_high() - AZ_get_low())) * 360.0;
}


/**
	Mjlighet att kra stlldon, etc. utan att bry sig om feedback. Enbart relstyrning.
	MENY-knappen ger avslut, vriga knappar styr reler.
*/
void start_freerunning (uint8_t print_pulses) {
uint8_t UP_pressed_before = 0;
uint8_t DOWN_pressed_before = 0;
uint8_t RIGHT_pressed_before = 0;
uint8_t LEFT_pressed_before = 0;
uint16_t el_pulses_save = 0;
uint8_t az_pulses_save = 0;
uint8_t el_stopped = 1;
uint8_t az_stopped = 1;
uint8_t	temp = 0;

	while (temp != _BV(MENU_B)) {
		temp = Knapptest();
		if ((print_pulses == PRINT_PULSES_ON) && (el_pulses_save != (uint8_t) EL_get_position())) {  //el_pulses)) {
			lcd_goto(LINE2+17);
			lcd_printint((uint8_t) EL_get_position());
			el_pulses_save = (uint8_t) EL_get_position();
		}
		if ((print_pulses == PRINT_PULSES_ON) && (az_pulses_save != (uint8_t) AZ_get_position())) {  //az_pulses)) {
			lcd_goto(LINE2+17);
			lcd_printint((uint8_t) AZ_get_position());
			az_pulses_save = (uint8_t) AZ_get_position();
		}
		
		/*	Nr en knapp trycks ner startas respektive motor (g till ndlge)
			Nr en knapp slpps stannas respektive motor (stopp!)
			
			Antingen upp ELLER ner, hger ELLER vnster. Men upp OCH hger (etc) r tilltet samtidigt
		*/
		if ((temp & _BV(UP_B)) != 0) {
			if (UP_pressed_before == 0) { 
				EL_set_position(0xFFFF); 
				UP_pressed_before = 1;
				el_stopped = 0;
			}
		}
		else if ((temp & _BV(DOWN_B)) != 0) {
			if (DOWN_pressed_before == 0) { 
				EL_set_position(0); 
				DOWN_pressed_before = 1;
				el_stopped = 0;
			}
		}
		
		if ((temp & _BV(RIGHT_B)) != 0) {
			if (RIGHT_pressed_before == 0) {
				AZ_set_position(0xFFFF);
				RIGHT_pressed_before = 1;
				az_stopped = 0;
			}
		}
		else if ((temp & _BV(LEFT_B)) != 0) {
			if (LEFT_pressed_before == 0) { 
				AZ_set_position(0); 
				LEFT_pressed_before = 1;
				az_stopped = 0;
			}
		}
		
		if (((temp & (_BV(UP_B) | _BV(DOWN_B))) == 0) && (el_stopped == 0)) {			// Om knapparna UPP och NER r slppta samtidigt s stannar vi
			EL_stop();
			el_stopped = 1;
			UP_pressed_before = 0;
			DOWN_pressed_before = 0;
		}
		if (((temp & (_BV(RIGHT_B) | _BV(LEFT_B))) == 0) && (az_stopped == 0)) {			// Om knapparna LEFT och RIGHT r slppta samtidigt s stannar vi
			AZ_stop();
			az_stopped = 1;
			RIGHT_pressed_before = 0;
			LEFT_pressed_before = 0;
		}
		
	}
}

/** 
	Hur man gr fr att kalibrera elevation
*/
void calibrate_el(void) {
uint16_t temp_min, temp_zero,temp_90, temp_max;

	EL_cal_start();

	lcd_cls();
	lcd_putstr("Set new EL low stop");
	lcd_goto(LINE2); lcd_putstr("Press MENU to enter");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());					// vnta p att knappen slpps
	temp_min = EL_get_position();
	
	lcd_goto(LINE1);lcd_putstr("Set new EL zero     ");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());
	temp_zero = EL_get_position();
	
	lcd_goto(LINE1); lcd_putstr("Set new EL 90       ");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());
	temp_90 = EL_get_position();
	
	lcd_goto(LINE1); lcd_putstr("Set new EL high stop");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());
	temp_max = EL_get_position();
	
	/* Spara */
	lcd_cls();
	if (EL_cal_write(temp_min, temp_zero, temp_90, temp_max)) lcd_putstr("Bad calibration!");
	else lcd_putstr("Calibration saved!");

	lcd_goto(LINE2); lcd_putstr("Press MENU to exit");
	Knapp_snabbtest();
}



/** 
	Hur man gr fr att kalibrera azimut
*/
void calibrate_az(void) {
uint16_t temp_min,temp_zero,temp_360, temp_max;
	
	AZ_cal_start();

	lcd_cls();
	lcd_putstr("Set new AZ low stop");
	lcd_goto(LINE2); lcd_putstr("Press MENU to enter");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());					// vnta p att knappen slpps
	temp_min = AZ_get_position();
	
	lcd_goto(LINE1);lcd_putstr("Set new AZ zero     ");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());
	temp_zero = AZ_get_position();
	
	lcd_goto(LINE1); lcd_putstr("Set new AZ 360       ");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());
	temp_360 = AZ_get_position();
	
	lcd_goto(LINE1); lcd_putstr("Set new AZ high stop");
	start_freerunning(PRINT_PULSES_ON);
	while (Knapptest());
	temp_max = AZ_get_position();

	// Giltighetskontroll:
	lcd_cls();
	if (AZ_cal_write(temp_min, temp_zero, temp_360, temp_max)) lcd_putstr("Bad calibration!");
	else lcd_putstr("Calibration saved!");
	
	lcd_goto(LINE2); lcd_putstr("Press MENU to exit");
	Knapp_snabbtest();
}



/**
	Skriver ut vart vi r pvg p den frsta raden p displayen

void Print_goto_angle(void) {
char temp[7];
	lcd_goto(LINE1);
	lcd_putstr("Go                  ");
	lcd_goto(LINE1+3); lcd_putch('['); double_to_string(get_AZ_target_angle(),temp,1); lcd_putstr(temp);  lcd_putstr("]    ");
	lcd_putch('['); double_to_string(get_EL_target_angle(),temp,1); lcd_putstr(temp); lcd_putch(']'); 
}
*/

/**
	Menyhantering
	*/
void Start_user_menu (void) {
enum meny_typ {CAL_AZ, CAL_EL, OFFSET_MODE, FREE} meny;
uint8_t continue_loop = 1;
uint8_t temp;
	
	while (Knapptest() != 0);								// vnta tills knappen har slppts
	
	meny = FREE;
	while (continue_loop == 1) {
		//temp = Knapp_snabbtest();
		switch (meny) {
			case CAL_AZ:
				lcd_cls();
				lcd_putstr("Calibrate AZ?");
				// testa knapp, loopa till UP,DWN eller ENT trycks ner	
				temp = Knapp_snabbtest();
				switch (temp) {							// analysera knapptryckningen
					case _BV(RIGHT_B):
						calibrate_az();
						break;
					case _BV(UP_B):
						meny = FREE;
						break;
					case _BV(DOWN_B):
						meny = CAL_EL;
						break;
					case _BV(MENU_B):
						continue_loop = 0;
						break;	
					default:								// denna fngar alla samtidiga knapptryckningar
						break;
				}	
				break;
			case CAL_EL:
				lcd_cls();
				lcd_putstr("Calibrate EL?");
				// testa knapp, loopa till UP,DWN eller ENT trycks ner
				temp = Knapp_snabbtest();
				switch (temp) {							// analysera knapptryckningen
					case _BV(RIGHT_B):
						calibrate_el();
						break;
					case _BV(UP_B):
						meny = CAL_AZ;
						break;
					case _BV(DOWN_B):
						meny = OFFSET_MODE;
						break;
					case _BV(MENU_B):
						continue_loop = 0;
						break;	
					default:								// denna fngar alla samtidiga knapptryckningar
						break;
				}	
				break;
			case OFFSET_MODE:
				lcd_cls();
				lcd_putstr("Offset mode");
				// testa knapp, loopa till UP,DWN eller ENT trycks ner
				temp = Knapp_snabbtest();
				
				switch (temp) {							// analysera knapptryckningen
					case _BV(RIGHT_B):
						// start_offset_mode();
						lcd_goto(LINE2);
						lcd_putstr("Ej implementerat");
						break;
					case _BV(UP_B):
						meny = CAL_EL;
						break;
					case _BV(DOWN_B):
						meny = FREE;
						break;
					case _BV(MENU_B):
						continue_loop = 0;
						break;	
					default:								// denna fngar alla samtidiga knapptryckningar
						break;
				}	
				break;
			case FREE:
				lcd_cls();
				lcd_putstr("Freerunning mode");
				// testa knapp, loopa till UP,DWN eller ENT trycks ner
				temp = Knapp_snabbtest();
				
				switch (temp) {							// analysera knapptryckningen
					case _BV(RIGHT_B):
						lcd_cls();
						lcd_putstr("-=Freerunning mode=-");
						lcd_goto(LINE2); lcd_putstr("Press MENU to exit");
						start_freerunning(PRINT_PULSES_OFF);			//_OFF
						break;
					case _BV(UP_B):
						meny = OFFSET_MODE;
						break;
					case _BV(DOWN_B):
						meny = CAL_AZ;
						break;
					case _BV(MENU_B):
						continue_loop = 0;
						break;	
					default:								// denna fngar alla samtidiga knapptryckningar
						break;
				}	
				break;				
			default:
				meny = OFFSET_MODE;								// Fngar alla oknda menyalternativ
				break;
		}
	}
}


/**
	Manuell kontroll
	*/
void Start_manual_control (void) {
uint16_t exit_loop = 400;
uint8_t last_button_held, hold_test, temp, first_az, first_el;
static int16_t az_angle_int, el_angle_int;
static uint16_t el_target, az_target;

	/*
	az_angle_int = (int16_t) get_AZ_target_angle();			// Ta heltalsdelen av 30,9 ger 30
	el_angle_int = (int16_t) get_EL_target_angle();
	el_target = el_pulses;
	az_target = az_pulses;
	*/

	if (!EL_in_motion()) {
		el_target = EL_get_position();
		el_angle_int = (int16_t) pulses_to_angle_el(EL_get_position());	//get_EL_angle();
	} // annars anvnder vi gamla vrden

	if (!AZ_in_motion()) {
		az_target = AZ_get_position();;
		az_angle_int = (int16_t) pulses_to_angle_az(AZ_get_position());	//get_AZ_angle();
	} // annars anvnder vi gamla vrden

	lcd_goto(LINE1); lcd_putstr("Go                  ");
	
	first_az = 1;							// om man i exemplet ovan (30,9) brjar med att trycka p vnster
	first_el = 1;							// kommer programmet att skriva 29 som frsta siffra. Tv extra variabler
											// fixar biffen!
	do {
		temp = Knapptest();
		if (temp != 0) {
			// detta sker nr knappen har tryckts ner:
			last_button_held = temp;
			hold_test = 60;
			do {
				temp = Knapptest ();
				if (temp != 0) {
					last_button_held = temp;
					hold_test--;
				}
			} while ((temp != 0) && (hold_test != 0));
			
			// testa fr meny:
			if (last_button_held != _BV(MENU_B)) {
				// hlls knappen nere?
				if (hold_test == 0) {
					// ja, knappen hlls nere
					switch (last_button_held) {
						case _BV(RIGHT_B):
							AZ_stop();
							while (Knapptest() == _BV(RIGHT_B)) {
								// lite frdrjning s att inte siffrorna rknar fr fort
								_delay_ms(32);
								_delay_ms(20);
								if (first_az == 1) first_az = 0;
								az_angle_int += 1;
								//set_new_AZ_angle((double) az_angle_int);
								az_target = AZ_angle_to_pulses((double) az_angle_int);
								lcd_goto(LINE1+3); lcd_putch('[');lcd_printint(az_angle_int);lcd_putstr("]  ");
							}
							break;
						case _BV(LEFT_B):
							AZ_stop();
							while (Knapptest() == _BV(LEFT_B)) {
								// lite frdrjning s att inte siffrorna rknar fr fort
								_delay_ms(32);
								_delay_ms(20);
								if (first_az == 1) first_az = 0;
								else az_angle_int -= 1; 
								//set_new_AZ_angle((double) az_angle_int);
								az_target = AZ_angle_to_pulses((double) az_angle_int);
								lcd_goto(LINE1+3); lcd_putch('[');lcd_printint(az_angle_int);lcd_putstr("]  ");
							}
							break;
						case _BV(UP_B):
							EL_stop();
							while (Knapptest() == _BV(UP_B)) {
								// lite frdrjning s att inte siffrorna rknar fr fort
								_delay_ms(32);
								_delay_ms(20);
								if (first_el == 1) first_el = 0;
								el_angle_int += 1;
								//set_new_EL_angle((double) el_angle_int);
								el_target = EL_angle_to_pulses((double) el_angle_int);
								lcd_goto(LINE1+13); lcd_putch('[');lcd_printint(el_angle_int);lcd_putstr("]  ");
							}
							break;
						case _BV(DOWN_B):
							EL_stop();
							while (Knapptest() == _BV(DOWN_B)) {
								// lite frdrjning s att inte siffrorna rknar fr fort
								_delay_ms(32);
								_delay_ms(20);
								if (first_el == 1) first_el = 0;
								else el_angle_int -= 1;
								//set_new_EL_angle((double) el_angle_int);
								el_target = EL_angle_to_pulses((double) el_angle_int);
								lcd_goto(LINE1+13); lcd_putch('[');lcd_printint(el_angle_int);lcd_putstr("]  ");
							}
							break;
						default:
							break;
					}
				}
				else {
					// knappen har slppts
					
					switch (last_button_held) {
						case _BV(RIGHT_B):
							AZ_stop();
							while (AZ_in_motion());
							if (first_az == 1) first_az = 0;
							az_angle_int += 1;
							//set_new_AZ_angle((double) az_angle_int); 
							az_target = AZ_angle_to_pulses((double) az_angle_int);
							lcd_goto(LINE1+3); lcd_putch('[');lcd_printint(az_angle_int);lcd_putstr("]  ");
							break;
						case _BV(LEFT_B):
							AZ_stop();
							while (AZ_in_motion());
							if (first_az == 1) first_az = 0;
							else az_angle_int -= 1; 
							//set_new_AZ_angle((double) az_angle_int);
							az_target = AZ_angle_to_pulses((double) az_angle_int);
							lcd_goto(LINE1+3); lcd_putch('[');lcd_printint(az_angle_int);lcd_putstr("]  ");
							break;
						case _BV(UP_B):
							EL_stop();
							while (EL_in_motion());
							if (first_el == 1) first_el = 0;
							el_angle_int += 1;
							//set_new_EL_angle((double) el_angle_int);
							el_target = EL_angle_to_pulses((double) el_angle_int);
							lcd_goto(LINE1+13); lcd_putch('[');lcd_printint(el_angle_int);lcd_putstr("]  ");
							break;
						case _BV(DOWN_B):
							EL_stop();
							while (EL_in_motion());
							if (first_el == 1) first_el = 0;
							else el_angle_int -= 1; 
							//set_new_EL_angle((double) el_angle_int);
							el_target = EL_angle_to_pulses((double) el_angle_int);
							lcd_goto(LINE1+13); lcd_putch('[');lcd_printint(el_angle_int);lcd_putstr("]  ");
							break;
						default:
							break;
					}
				}
			}
			exit_loop = 400;
		}
	} while (--exit_loop > 0);
	
	EL_set_position(el_target);
	AZ_set_position(az_target);
}



/**
	skriv ut det aktuella gradtalet, AZ
*/
void print_az_degrees (void) {
char temp_str[7];
	lcd_goto(LINE2+4);
	double_to_string(pulses_to_angle_az(AZ_get_position()),temp_str,1);
	lcd_putstr(temp_str);
	lcd_putstr("  ");
}

/**
	skriv ut det aktuella gradtalet, EL
*/
void print_el_degrees (void) {
char temp_str[7];
	lcd_goto(LINE2+14);
	double_to_string(pulses_to_angle_el(EL_get_position()),temp_str,1);
	lcd_putstr(temp_str);
	lcd_putstr("  ");
}

/** 
	Skriv ut standardtexten
*/

void print_standard_text(void) {
char temp_str[7];
	lcd_cls();
	lcd_putstr("SatFinder by SM6YVS");
	lcd_goto(LINE2); lcd_putstr("AZ: ");
	double_to_string(pulses_to_angle_az(AZ_get_position()),temp_str,1);
	lcd_putstr(temp_str);
	lcd_goto(LINE2+10); lcd_putstr("EL: "); 
	double_to_string(pulses_to_angle_el(EL_get_position()),temp_str,1); 
	lcd_putstr(temp_str);
}

/**
	Hantering av meddelanden frn ngon styrmojng (PC)
	
	Protokollet imiterar Alpa Spid az/el rotor.
*/
void process_msg(void) {
double double_temp, double_heltalsdel;
static uint8_t last_az_ascii[5];
static uint8_t last_el_ascii[5];
#	ifdef USART_debug
	while ((UCSR0A & _BV(UDRE0)) == 0);
	UDR0 = RX_buffer[0];											// debug msg
	while ((UCSR0A & _BV(UDRE0)) == 0);
	UDR0 = RX_buffer[12];											// debug msg
#	endif

	// Fortstt enbart om ramen r korrekt
	if ((RX_buffer[0] == 0x57) && (RX_buffer[12] == 0x20)) {
#		ifdef USART_debug
		while ((UCSR0A & _BV(UDRE0)) == 0);
		UDR0 = 's';											// debug msg
#		endif
	
		switch (RX_buffer[11]) {
			case 0x0F:						// Stop
				EL_stop();
				AZ_stop();
				/*	Efter ett stop-kommando frn PC s skall vi skicka
					tillbaka den verkliga positionen */
				Ange_verklig_position = 1;
				
				// fall igenom:
			case 0x1F:						// Status
#				ifdef USART_debug
				while ((UCSR0A & _BV(UDRE0)) == 0);
				UDR0 = '1';											// debug msg
#				endif			
				TX_buffer[0] = 0x57;
				
				if (Ange_verklig_position) {
					/* Avrunda till .5 graders nogrannhet - funkar bara inte!
						double_temp = modf((get_AZ_angle() + 360), &double_heltalsdel);		
						if (double_temp > 0.75) 
							double_heltalsdel += 1.0;
						if ((double_temp <= 0.75) && (double_temp > 0.25)) 
							double_temp = 0;
						else 
							double_temp = 0.5;
						double_temp = double_temp + double_heltalsdel;
					*/
					/* Avrunda till EN grads nogrannhet - funkar det? */
					double_temp = pulses_to_angle_az(AZ_get_position()) + 360;
					if ((modf((pulses_to_angle_az(AZ_get_position()) + 360), &double_heltalsdel)) >= 0.5)
						double_temp += 1.0;
					double_to_string(double_temp,&TX_buffer[1],1);
					TX_buffer[4] = 0;		//TX_buffer[5];		// Skriv ver punkten!
					TX_buffer[5] = 1;		//2;				// 2 = 0,5 grad per impuls, 1 = 1 grad per impuls
				}
				else {
					for (uint8_t i = 0; i < 5; i++) TX_buffer[i+1] = last_az_ascii[i];
				}
				
				if (Ange_verklig_position) {
					/* Avrunda till .5 graders nogrannhet - funkar allts inte...
						double_temp = modf((get_EL_angle() + 360), &double_heltalsdel);		
						if (double_temp > 0.75) double_heltalsdel += 1.0;
						if ((double_temp > 0.75) || (double_temp <= 0.25)) double_temp = 0;
						else double_temp = 0.5;
						double_temp += double_heltalsdel;
					*/
					/* Avrunda till EN grads nogrannhet - funkar det? */
					double_temp = pulses_to_angle_el(EL_get_position()) + 360;
					if ((modf((pulses_to_angle_el(EL_get_position()) + 360), &double_heltalsdel)) >= 0.5)
						double_temp += 1.0;
						
					double_to_string(double_temp,&TX_buffer[6],1);
					TX_buffer[9] = 0;		//TX_buffer[10];		// Skriv ver punkten!
					TX_buffer[10] = 1;		//2;
				}
				else {
					for (uint8_t i = 0; i < 5; i++) TX_buffer[i+6] = last_az_ascii[i];
				}
				
				TX_buffer[11] = 0x20;
				
				Send_message();
				break;
			case 0x2F:						// Set
			
			/* Hr mste vi stoppa el och az innan vi gr vidare?! */
				EL_stop();
				AZ_stop();
				while (EL_in_motion());									// vnta tills stlldonet har stannat
				while (AZ_in_motion());
				
				/* Vi svarar en frgande PC att vi r p den NSKADE positionen */
				Ange_verklig_position = 0;
				
				/* Kopiera den nskade positionen samt dividend */
				for (uint8_t i = 0; i < 5; i++) last_az_ascii[i] = RX_buffer[i+1];
				
				double_temp = (RX_buffer[1] - 0x30) * 1000;						// tusental 
				double_temp += (RX_buffer[2] - 0x30) * 100;						// hundratal 
				double_temp += (RX_buffer[3] - 0x30) * 10;						// tiotal 
				double_temp += (RX_buffer[4] - 0x30);								// ental
				if (RX_buffer[5] == 0x02) double_temp /= 2;						// dividera med tv
				double_temp -= 360;												// ta bort 360 graders offset
				AZ_set_position(AZ_angle_to_pulses (double_temp));
				
				/* Kopiera den nskade positionen samt dividend */
				for (uint8_t i = 0; i < 5; i++) last_el_ascii[i] = RX_buffer[i+6];
				
				double_temp = (RX_buffer[6] - 0x30) * 1000;						// tusental 
				double_temp += (RX_buffer[7] - 0x30) * 100;						// hundratal 
				double_temp += (RX_buffer[8] - 0x30) * 10;						// tiotal 
				double_temp += (RX_buffer[9] - 0x30);								// ental
				if (RX_buffer[10] == 0x02) double_temp /= 2;						// dividera med tv
				double_temp -= 360;												// ta bort 360 graders offset
				EL_set_position(EL_angle_to_pulses (double_temp));
				
				break;
			default:
#				ifdef USART_debug
				while ((UCSR0A & _BV(UDRE0)) == 0);
				UDR0 = 0xEE;											// debug msg
#				endif						
				break;
		}
	}
}


/**
	Huvudprogrammet
*/
int main (void) {
uint8_t temp, time_to_save_ee;
uint16_t az_pulses_old,el_pulses_old;
	
	io_init();
	lcd_init();	

	print_standard_text();
	
	/* Aktivera USART-mottagning */
	UCSR0B |= _BV(RXCIE0);
	
	// Huvudloopen. Kontrollerar a) anvndarinput, b) om det skall skrivas ut nya gradtal.
	for (;;) {
		// a) Kontrollera anvndarinput
		temp = Knapptest();
		if (temp != 0) {
			if (temp == _BV(MENU_B)) {
				Start_user_menu();
				/*	I User_menu kan man anvnda offset, drfr lter vi 
					"Ange_verklig_position" vara ofrndrad */
			} else {
				//AZ_stop(); az_target = az_pulses;
				//EL_stop(); el_target = el_pulses;
				Start_manual_control();
				/* Efter att vi har krt rotorn manuellt skall vi snda verkligt data */
				Ange_verklig_position = 1;
			}
			// Skriv ut bastexten p displayen
			print_standard_text();
			
			// nolla spar-tiden:
			time_to_save_ee = 0;
		}
		// b) kontrollera om det skall skrivas ut nya gradtal
		if ((TCCR0B != 0) || (AZ_get_position() != az_pulses_old)){
			print_az_degrees();
			az_pulses_old = AZ_get_position();
			}
		if ((TCCR1B != 0) || (EL_get_position() != el_pulses_old)) {
			print_el_degrees();
			el_pulses_old = EL_get_position();
			}
		/* Kolla om det kommit ngra nya meddelanden via USART */
		if (NEW_USART_MSG) {
			process_msg();
			NEW_USART_MSG = 0;
			}
	}
}




/**
	PC interface. spara inkommande tecken etc.
	*/
ISR(USART_RX_vect)
{
static uint8_t index;
uint8_t temp;

	temp = UDR0;											// ls inkommande tecken
#	ifdef USART_debug
	UDR0 = temp;											// echo!
#	endif
	if (temp == 0x57)  index = 0;							// tecken 0x57 r starttecken
	if (index < 13) RX_buffer[index] = temp;				// spara i buffern

	if (++index >= 13) { 									// ett fullstndigt meddelande innehller 13 tecken
		NEW_USART_MSG = 1;
		index = 0;
	}
	else NEW_USART_MSG = 0;

}

/** 
	Skicka ett tecken t gngen. Nr alla 12 tecknen har skickats stngs avbrotten av.
*/
ISR(USART_UDRE_vect)
{	
	UDR0 = TX_buffer[TX_index++];
	if (TX_index >= 12) UCSR0B &= ~_BV(UDRIE0);		// stng av fortsatta interrupts
}


/**
	Fnga buggiga interrupts
*/
ISR(__vector_default)
{
    lcd_cls();
	lcd_putstr("Interrupt bug!");
	PORTD = 0;
	while (Knapptest() != 0);
	while (Knapptest() == 0);
	while (Knapptest() != 0);
}

